;----------------------------------------------------------------------
; pause.asm - routines for fixed rate time delays from 500us to 1s
;
; Call		Cycles in call and return (4)
; Setup		cycles to initialize from loop (2)
; Meat		Number of cycles wasted inside loop
; Pass		Cycles to loop (3)
; Last		Cycles to fall through last iteration (2)
; Count		Number of loop iterations
; 
; Single-nested loops use the formula:
;  Cycles = Call + Setup + (Count - 1) * (Meat + Pass) + (Meat + Last)
;
; Double-nested loops use the formulas:
;  Inner = Setup + (Count - 1) * (Meat + Pass) + (Meat + Last)
;  Cycles = Call + Setup + (Count - 1) * (Inner + Pass) + (Inner + Last)
;
;----------------------------------------------------------------------

	cblock
pause_i
pause_j
pause_k
	endc

;----------------------------------------------------------------------
; Pause 500 uS

pause_500us	
#ifdef debug
		return
#endif
		from	pause_i, .165	; 3 cycles for init
		endfrom			; 3 cycle

		return

;----------------------------------------------------------------------
; Pause 1 mS

pause_1ms	
#ifdef debug
		return
#endif
		from	pause_i, .199
		 goto $+1
		endfrom

		return

;----------------------------------------------------------------------
; Pause 10mS

pause_10ms
#ifdef debug
		return
#endif
		from	pause_i, .42
		 from	pause_j, .78
		 endfrom
		endfrom

		return

;----------------------------------------------------------------------

pause_50ms
#ifdef debug
		return
#endif
		from	pause_i, .99
		 from	pause_j, .167
		 endfrom
		endfrom

		return

;----------------------------------------------------------------------

pause_100ms	call	pause_50ms
		call	pause_50ms

		return

;----------------------------------------------------------------------
; Pause 1 Second

pause_1s	from	pause_k, .10
		 call	pause_100ms
		endfrom

		return

